package com.jiuqi.syntax;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.Map;

public final class AutoFunctionListener extends CustomFunctionListener {
   private Object provider;
   private String classify;
   private Map funcMap;
   private static final int SY_FUNC_CALL_ERROR = 10400;
   private static final int SY_FUNC_RET_ERROR = 10401;
   private static final int SY_FUNC_ACL_ERROR = 10402;

   public AutoFunctionListener(Object provider) {
      this(provider, (String)null);
   }

   public AutoFunctionListener(Object provider, String classify) {
      this.provider = provider;
      this.classify = classify;
      this.initFunctions();
   }

   private void initFunctions() {
      this.funcMap = new Hashtable();
      if (this.provider != null) {
         Class pc = this.provider.getClass();
         Method[] methods = pc.getDeclaredMethods();

         label36:
         for(int i = 0; i < methods.length; ++i) {
            if (Modifier.isPublic(methods[i].getModifiers())) {
               Function func = new Function();
               func.name = methods[i].getName();
               func.resultType = this.convertType(methods[i].getReturnType());
               if (func.resultType != 0) {
                  func.classify = this.classify;
                  Class[] paramTypes = methods[i].getParameterTypes();

                  for(int j = 0; j < paramTypes.length; ++j) {
                     FunctionParam param = new FunctionParam();
                     param.name = "p" + Integer.toString(j);
                     param.type = this.convertType(paramTypes[j]);
                     if (param.type == 0) {
                        continue label36;
                     }

                     param.canOmit = false;
                     param.isSign = false;
                     func.params.add(param);
                  }

                  this.functions.add(func);
                  this.funcMap.put(func, methods[i]);
               }
            }
         }

      }
   }

   private int convertType(Class typeClass) {
      if (!Integer.TYPE.equals(typeClass) && !Integer.class.equals(typeClass)) {
         if (!Double.TYPE.equals(typeClass) && !Double.class.equals(typeClass)) {
            if (String.class.equals(typeClass)) {
               return 3;
            } else if (!Boolean.TYPE.equals(typeClass) && !Boolean.class.equals(typeClass)) {
               return Calendar.class.equals(typeClass) ? 6 : 0;
            } else {
               return 5;
            }
         } else {
            return 2;
         }
      } else {
         return 1;
      }
   }

   public String getClassify() {
      return this.classify;
   }

   public int callFunction(String funcName, CommonDatas paramValues, CommonData resultData, boolean isCheckMode) {
      Function func = this.functions.getFunc(funcName);
      if (func == null) {
         return 201;
      } else {
         Method method = (Method)this.funcMap.get(func);

         try {
            Object[] args = this.convertParams(paramValues, func);
            Object ret = method.invoke(this.provider, args);
            resultData.type = func.resultType;
            return !this.convertObject(ret, resultData) ? 10401 : 0;
         } catch (SyntaxException var9) {
            SyntaxException e = var9;
            return e.getErrorCode();
         } catch (IllegalArgumentException var10) {
            return 202;
         } catch (IllegalAccessException var11) {
            return 10402;
         } catch (InvocationTargetException var12) {
            InvocationTargetException e = var12;
            e.printStackTrace();
            return 10400;
         }
      }
   }

   private Object[] convertParams(CommonDatas paramValues, Function func) throws SyntaxException {
      Object[] params = new Object[func.params.size()];

      for(int i = 0; i < func.params.size(); ++i) {
         CommonData value = paramValues.size() <= i ? null : paramValues.getItem(i);
         if (value != null && !value.isNull) {
            switch (func.params.getParam(i).type) {
               case 1:
                  params[i] = new Integer((int)value.getIntRealValue());
                  break;
               case 2:
                  params[i] = new Double(value.getIntRealValue());
                  break;
               case 3:
                  params[i] = value.cdString;
                  break;
               case 4:
               default:
                  throw new SyntaxException(202);
               case 5:
                  params[i] = new Boolean(value.cdBool);
                  break;
               case 6:
                  params[i] = value.cdDate;
            }
         } else {
            params[i] = null;
         }
      }

      return params;
   }

   private boolean convertObject(Object value, CommonData result) {
      if (value == null) {
         result.isNull = true;
         result.initNullValue();
      } else {
         result.isNull = false;
         switch (result.type) {
            case 1:
               if (value instanceof Integer) {
                  result.cdInt = (Integer)value;
               } else {
                  if (!(value instanceof Double)) {
                     return false;
                  }

                  result.cdInt = ((Double)value).intValue();
               }
               break;
            case 2:
               if (value instanceof Integer) {
                  result.cdReal = (double)(Integer)value;
               } else {
                  if (!(value instanceof Double)) {
                     return false;
                  }

                  result.cdReal = (Double)value;
               }
            case 3:
               result.cdString = value.toString();
               break;
            case 4:
            default:
               return false;
            case 5:
               if (!(value instanceof Boolean)) {
                  return false;
               }

               result.cdBool = (Boolean)value;
               break;
            case 6:
               if (value instanceof Calendar) {
                  result.cdDate = (Calendar)value;
               } else if (value instanceof Date) {
                  result.setDateValue((Date)value);
               } else {
                  if (!(value instanceof Long)) {
                     return false;
                  }

                  Calendar date = Calendar.getInstance();
                  date.setTimeInMillis((Long)value);
                  result.cdDate = date;
               }
         }
      }

      return true;
   }

   public String getCodeMessage(int retCode) {
      switch (retCode) {
         case 10400:
            return "函数调用时出现程序异常";
         case 10401:
            return "错误的函数返回值类型";
         case 10402:
            return "类或函数无法访问";
         default:
            return null;
      }
   }

   private class SyntaxException extends Exception {
      private int errorCode;

      public SyntaxException(int errorCode) {
         this.errorCode = errorCode;
      }

      public int getErrorCode() {
         return this.errorCode;
      }
   }
}
